<?php
defined( 'ABSPATH' ) || exit;

class Better_Messages_Shortcodes
{

    public static function instance()
    {

        // Store the instance locally to avoid private static replication
        static $instance = null;

        // Only run these methods if they haven't been run previously
        if ( null === $instance ) {
            $instance = new Better_Messages_Shortcodes;
            $instance->setup_actions();
        }

        // Always return the instance
        return $instance;

        // The last metroid is in captivity. The galaxy is at peace.
    }

    public function setup_actions(){
        add_shortcode( 'bp_better_messages_unread_counter', array( $this, 'unread_counter_shortcode' ) );
        add_shortcode( 'better_messages_unread_counter', array( $this, 'unread_counter_shortcode' ) );

        add_shortcode( 'bp_better_messages_my_messages_url', array( $this, 'bp_better_messages_url' ) );
        add_shortcode( 'better_messages_my_messages_url', array( $this, 'bp_better_messages_url' ) );

        add_shortcode( 'bp_better_messages_pm_button', array( $this, 'bp_better_messages_pm_button' ) );
        add_shortcode( 'better_messages_pm_button', array( $this, 'bp_better_messages_pm_button' ) );

        add_shortcode( 'bp_better_messages', array( $this, 'bp_better_messages' ) );
        add_shortcode( 'better_messages', array( $this, 'bp_better_messages' ) );

        /**
         * Premium buttons
         */
        add_shortcode( 'bp_better_messages_mini_chat_button',  array( $this, 'bp_better_messages_mini_chat_button' ) );
        add_shortcode( 'better_messages_mini_chat_button',  array( $this, 'bp_better_messages_mini_chat_button' ) );

        add_shortcode( 'bp_better_messages_video_call_button', array( $this, 'bp_better_messages_video_call_button' ) );
        add_shortcode( 'better_messages_video_call_button', array( $this, 'bp_better_messages_video_call_button' ) );

        add_shortcode( 'bp_better_messages_audio_call_button', array( $this, 'bp_better_messages_audio_call_button' ) );
        add_shortcode( 'better_messages_audio_call_button', array( $this, 'bp_better_messages_audio_call_button' ) );

        add_shortcode( 'better_messages_single_conversation', array( $this, 'better_messages_single_conversation' ) );
    }

    public function better_messages_single_conversation( $args ){
        $thread_id = intval( $args['thread_id'] );
        $thread = Better_Messages()->functions->get_thread( $thread_id );

        if( $thread ) {
            return Better_Messages()->functions->get_conversation_layout($thread_id);
        } else {
            return '<p>' . __('Conversation not exists', 'bp-better-messages') .  '</p>';
        }
    }

    public function bp_better_messages(){
        ob_start();

        if( ! is_user_logged_in() ){
            echo Better_Messages()->functions->render_login_form();
        } else {
            if (function_exists('bp_is_user') && bp_is_user()) {
                echo Better_Messages()->functions->get_page();
            } else {
                echo Better_Messages()->functions->get_page(true);
            }
        }

        return ob_get_clean();
    }

    public function bp_better_messages_pm_button( $args ){
        if( ! is_user_logged_in() ){
            return '';
        }

        $class   = 'bpbm-pm-button';
        $target  = '';
        $text    = __('Private Message', 'bp-better-messages');
        $subject = '';
        $message = '';
        $fast    = true;
        $return_url = false;

        if( isset( $args['class'] ) ) {
            $class .= ' ' . $args['class'];
        }

        if( isset( $args['target'] ) ) {
            $target .= ' target="' . $args['target'] . '"';
        }

        if( isset( $args['text'] ) ) {
            $text = $args['text'];
        }

        if( isset( $args['subject'] ) ) {
            $subject = urlencode($args['subject']);
        }

        if( isset( $args['message'] ) ) {
            $message = urlencode($args['message']);
        }

        if( isset( $args['fast_start'] ) && $args['fast_start'] === '0' ) {
            $fast = false;
        }

        if( isset( $args['url_only'] ) && $args['url_only'] === '1' ) {
            $return_url = true;
        }

        if( isset( $args['user_id'] ) ) {
            $user_id = (int) $args['user_id'];
        } else {
            $user_id = (int) Better_Messages()->functions->get_member_id();
        }

        if( $user_id === get_current_user_id() ) $class .= ' bm-self-button';

        $userdata = get_userdata( $user_id );
        if( ! $userdata ) return '';


        $args = [
            'to' => $user_id
        ];

        $base_url = Better_Messages()->functions->get_link(get_current_user_id());

        if( Better_Messages()->settings['fastStart'] == '1' && $fast ){
            $args['bm-fast-start'] = '1';
        }

        if( ! empty( $subject ) ){
            $args['subject'] = $subject;
        }

        if( ! empty( $message ) ){
            $args['message'] = $message;
        }

        if( $args['bm-fast-start'] ){
            $link = add_query_arg( $args, $base_url );
        } else {
            $link = Better_Messages()->functions->add_hash_arg('new-conversation', $args, $base_url);
        }

        if( $return_url ) {
            return $link;
        }

        return '<a href="' . esc_url($link) .  '" class="' . esc_attr($class) . '"' . $target . '>' . esc_attr($text) . '</a>';
    }

    public function bp_better_messages_video_call_button( $args ){
        if( ! is_user_logged_in() ){
            return '';
        }

        $class   = 'bpbm-pm-button video-call';
        $target  = '';
        $text    = __('Video Call', 'bp-better-messages');
        $return_url = false;

        if( isset( $args['class'] ) ) {
            $class .= ' ' . $args['class'];
        }

        if( isset( $args['target'] ) ) {
            $target .= ' target="' . $args['target'] . '"';
        }

        if( isset( $args['text'] ) ) {
            $text = $args['text'];
        }

        if( isset( $args['url_only'] ) && $args['url_only'] === '1' ) {
            $return_url = true;
        }

        if( isset( $args['user_id'] ) ) {
            $user_id = (int) $args['user_id'];
        } else {
            $user_id = (int) Better_Messages()->functions->get_member_id();
        }

        if( $user_id === get_current_user_id() ) $class .= ' bm-self-button';

        $userdata = get_userdata( $user_id );
        if( ! $userdata ) return '';
        $args = [
            'fast-call' => '',
            'to' => $user_id,
            'type' => 'video'
        ];

        $base_url = Better_Messages()->functions->get_link( get_current_user_id() );
        $link = add_query_arg( $args, $base_url );

        if( $return_url ) {
            return $link;
        }

        return '<a href="' . esc_url($link) .  '" class="' . esc_attr($class) . '" data-user-id="' . $user_id . '">' . esc_attr($text) . '</a>';
    }

    public function bp_better_messages_audio_call_button( $args ){
        if( ! is_user_logged_in() ){
            return '';
        }

        $class   = 'bpbm-pm-button audio-call';
        $text    = __('Audio Call', 'bp-better-messages');
        $return_url = false;

        if( isset( $args['class'] ) ) {
            $class .= ' ' . $args['class'];
        }

        if( isset( $args['text'] ) ) {
            $text = $args['text'];
        }

        if( isset( $args['url_only'] ) && $args['url_only'] === '1' ) {
            $return_url = true;
        }

        if( isset( $args['user_id'] ) ) {
            $user_id = (int) $args['user_id'];
        } else {
            $user_id = (int) Better_Messages()->functions->get_member_id();
        }

        if( $user_id === get_current_user_id() ) $class .= ' bm-self-button';

        $userdata = get_userdata( $user_id );
        if( ! $userdata ) return '';

        $args = [
            'fast-call' => '',
            'to' => $user_id,
            'type' => 'audio'
        ];

        $base_url = Better_Messages()->functions->get_link(get_current_user_id());
        $link = add_query_arg( $args, $base_url );

        if( $return_url ) {
            return $link;
        }

        return '<a href="' . esc_url($link) .  '" class="' . esc_attr($class) . '" data-user-id="' . $user_id . '">' . esc_attr($text) . '</a>';
    }

    public function bp_better_messages_mini_chat_button( $args ){
        if( ! is_user_logged_in() ){
            return '';
        }

        if (Better_Messages()->settings['miniChatsEnable'] !== '1') {
            return '';
        }

        $class   = 'bpbm-pm-button open-mini-chat';
        $text    = __('Private Message', 'bp-better-messages');

        if( isset( $args['class'] ) ) {
            $class .= ' ' . $args['class'];
        }

        if( isset( $args['text'] ) ) {
            $text = $args['text'];
        }

        if( isset( $args['user_id'] ) ) {
            $user_id = (int) $args['user_id'];
        } else {
            $user_id = (int) Better_Messages()->functions->get_member_id();
        }

        if( $user_id === get_current_user_id() ) $class .= ' bm-self-button';

        $userdata = get_userdata( $user_id );
        if( ! $userdata ) return '';

        $attributes = '';

        if( isset( $args['subject'] ) ) {
            $attributes .= ' data-bm-subject="' . urlencode($args['subject']) . '"';
        }

        if( isset( $args['unique_tag'] ) ) {
            $attributes .= ' data-bm-unique-key="' . urlencode($args['unique_tag']) . '"';
        }

        $link = '#';

        return '<a href="' . esc_url($link) .  '" class="' . esc_attr($class) . '" data-user-id="' . $user_id . '" '. $attributes. '>' . esc_attr($text) . '</a>';
    }

    public function bp_better_messages_url(){
        if( ! is_user_logged_in() ){
            return '';
        }

        return Better_Messages()->functions->get_link( get_current_user_id() );
    }

    function unread_counter_shortcode( $args ) {
        if( ! is_user_logged_in() ){
            return '';
        }

        $hide_when_no_messages = false;
        $preserve_space = false;
        if( isset( $args['hide_when_no_messages'] ) && $args['hide_when_no_messages'] === '1' ) {
            $hide_when_no_messages = true;
        }

        if( isset( $args['preserve_space'] ) && $args['preserve_space'] === '1' ) {
            $preserve_space = true;
        }

        $classes = ['bp-better-messages-unread', 'bpbmuc'];
        if( $hide_when_no_messages ){
            $classes[] = 'bpbmuc-hide-when-null';
        }

        if( $preserve_space ){
            $classes[] = 'bpbmuc-preserve-space';
        }

        $class = implode(' ', $classes );
        if( Better_Messages()->settings['mechanism'] !== 'websocket'){
            $unread = Better_Messages()->functions->get_total_threads_for_user( get_current_user_id(), 'unread' );
            return '<span class="' . $class . '" data-count="' . $unread . '">' . $unread . '</span>';
        } else {
            return '<span class="' . $class . '" data-count="0">0</span>';
        }
    }

}

function Better_Messages_Shortcodes()
{
    return Better_Messages_Shortcodes::instance();
}
